# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-07 19:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ReadPermissions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=50)),
                ('namespace', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'Legacy read permission',
                'verbose_name_plural': 'Legacy read permissions',
            },
        ),
        migrations.CreateModel(
            name='WritePermissions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=50)),
                ('namespace', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'Legacy write permission',
                'verbose_name_plural': 'Legacy write permissions',
            },
        ),
    ]
